﻿using Microsoft.IdentityModel.Protocols.OpenIdConnect;
using Microsoft.IdentityModel.Tokens;
using Microsoft.Owin.Security.Cookies;
using Microsoft.Owin.Security.OpenIdConnect;
using Microsoft.Owin.Security;
using Owin;
using System.Linq;
using System.Threading.Tasks;
using System;
using System.Web;

namespace WisejWebPageApplicationOpenID
{
	public static class TestAuthenticationMiddleware
	{
		// client id from the azure AD application.
		static string clientId = "537de571-faa3-4cc4-9860-e66e9577288e";
		static string tenantId = "d196de67-f5c5-4763-b4e8-84aa6756e0c9";

		public static IAppBuilder UseTestAuthentication(this IAppBuilder app)
		{
			// tenant id from the azure AD application.
			// string tenant = args.TenantId;

			// authority is the URL for authority, composed of the Microsoft identity platform and the tenant name (e.g. https://login.microsoftonline.com/contoso.onmicrosoft.com/v2.0)
			string authority = $"https://sts.windows.net/{tenantId}/v2.0";

			// redirectUri is the URL where the user will be redirected to after they sign in.
			string redirectUri = "https://localhost:44391/authenticate/";

			app.SetDefaultSignInAsAuthenticationType(CookieAuthenticationDefaults.AuthenticationType);

			app.UseCookieAuthentication(new CookieAuthenticationOptions());

			app.UseOpenIdConnectAuthentication(
				new OpenIdConnectAuthenticationOptions
				{
					// Sets the ClientId, authority, RedirectUri as obtained from web.config
					ClientId = clientId,
					Authority = authority,
					RedirectUri = redirectUri,
					RequireHttpsMetadata = false,
					// PostLogoutRedirectUri is the page that users will be redirected to after sign-out. In this case, it is using the home page
					PostLogoutRedirectUri = redirectUri,
					Scope = "openid profile email",
					// ResponseType is set to request the code id_token - which contains basic information about the signed-in user
					ResponseType = OpenIdConnectResponseType.CodeIdToken,

					// ValidateIssuer set to false to allow personal and work accounts from any organization to sign in to your application
					// To only allow users from a single organizations, set ValidateIssuer to true and 'tenant' setting in web.config to the tenant name
					// To allow users from only a list of specific organizations, set ValidateIssuer to true and use ValidIssuers parameter
					TokenValidationParameters = new TokenValidationParameters()
					{
						ValidateIssuer = false // This is a simplification
					}
				});

			var mapAuthenticationUri = redirectUri.Split("/".ToArray(), StringSplitOptions.RemoveEmptyEntries);

			app.Map("/" + mapAuthenticationUri.Last(), (builder) =>
			{
				builder.Run((context) =>
				{
					context.Response.Redirect("/");
					return Task.CompletedTask;
				});
			});

			app.Map("/login", (builder) =>
			{
				builder.Run((context) =>
				{
					if (HttpContext.Current.Request.IsAuthenticated)
					{
						context.Response.Redirect("/");
					}
					else
					{
						context.Authentication.Challenge(
								new AuthenticationProperties { RedirectUri = redirectUri },
								OpenIdConnectAuthenticationDefaults.AuthenticationType);
					}

					return Task.CompletedTask;
				});
			});

			app.Map("/logout", (builder) =>
			{
				builder.Run((context) =>
				{
					if (HttpContext.Current.Request.IsAuthenticated)
					{
						context.Authentication.SignOut(
								OpenIdConnectAuthenticationDefaults.AuthenticationType,
								CookieAuthenticationDefaults.AuthenticationType);
					}
					else
					{
						context.Response.Redirect("/");
					}

					return Task.CompletedTask;
				});
			});
			return app;
		}
	}
}