﻿using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.OpenIdConnect;
using Microsoft.AspNetCore.Builder;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Identity.Web;
using System.Threading.Tasks;

namespace WisejWebPageApplicationOpenID
{
	public static class TestAuthenticationMiddleware
	{
		// client id from the azure AD application.
		static string clientId = "537de571-faa3-4cc4-9860-e66e9577288e";
		static string tenantId = "d196de67-f5c5-4763-b4e8-84aa6756e0c9";

		public static IServiceCollection AddTestAuthentication(this IServiceCollection services)
		{
			services
				.AddAuthentication(OpenIdConnectDefaults.AuthenticationScheme)
				.AddMicrosoftIdentityWebApp((options) =>
				{
					options.Instance = "https://login.microsoftonline.com/";
					options.TenantId = tenantId;
					options.ClientId = clientId;
					options.CallbackPath = "/authenticate/";
				});

			return services;
		}

		public static WebApplication UseTestAuthorization(this WebApplication app)
		{
			app.Map("/login", (context) =>
			{
				if (context.User?.Identity.IsAuthenticated == true)
				{
					context.Response.Redirect("/");
				}
				else{
					return context.ChallengeAsync();
				}
				return Task.CompletedTask;
			});

			app.Map("/logout", (context) =>
			{
				if (context.User?.Identity.IsAuthenticated == true)
				{
					context.Response.Cookies.Delete(".AspNetCore.Cookies");
					return context.SignOutAsync(OpenIdConnectDefaults.AuthenticationScheme, new AuthenticationProperties
					{
						RedirectUri = "/"
					});
				}
				else
				{
					context.Response.Redirect("/");
					return Task.CompletedTask;
				}
			});


			return app;
		}
	}
}
